#pragma once
#include <iostream>
#include <sstream>
#include <string>
#include <utility>
#include <chrono>

template<class ...Args>
std::string fmt(Args&&...args){
    std::ostringstream oss;
    (oss<<...<<std::forward<Args>(args));
    return std::move(std::string(oss.str()));
}

template<class ...Args>
void print(Args...args){
    std::ostringstream oss;
    (oss<<...<<std::forward<Args>(args));
    std::cout<<oss.str();
}

#define NVC(v)#v," : ",(v)

class Spent{ // 使用RAII原理的自动计时器，计算主函数运行时间，析构时自动输出
    private:
        const decltype(std::chrono::system_clock::now()) start;
        const std::string name;
    public:
        Spent(const std::string name)noexcept:start(std::chrono::system_clock::now()),name(name){
            print("[Timer: ",name,"]"," Start timing","\n");
        }
        ~Spent()noexcept{
            const auto end = std::chrono::system_clock::now();
            const auto dur = std::chrono::duration_cast<std::chrono::milliseconds> (end-start);
            print("[Timer: ",name,"]"," Stop timing , used ", dur.count(),"ms\n");
        }
    };